<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2020 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class AvisAuthentiques extends Module
{
    protected $config_form = false;

//    protected $baseURL = 'https://api.avis-authentiques.fr/api';
    protected $baseURL = 'https://vulcain/api';

    public function __construct()
    {
        $this->name = 'avisauthentiques';
        $this->tab = 'advertising_marketing';
        $this->version = '1.0.0';
        $this->author = 'Avis-Authentiques.fr';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Avis-Authentiques');
        $this->description = $this->l('Collect customer\'s reviews and implement widget from Avis-Authentiques.fr');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);

        $this->_errors= ['dfdgfdg'];
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('AVIS-AUTHENTIQUES_LIVE_MODE', false);
        Configuration::updateValue('AVIS-AUTHENTIQUES_POSITION', 'middle');

        return parent::install() &&
            $this->registerHook('actionPaymentConfirmation') &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
//            $this->registerHook('displayLeftColumn') &&
            $this->registerHook('displayBeforeBodyClosingTag') &&
            $this->registerHook('displayFooter');


    }

    public function uninstall()
    {
        Configuration::deleteByName('AVIS-AUTHENTIQUES_LIVE_MODE');
        Configuration::deleteByName('AVIS-AUTHENTIQUES_POSITION');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $output = '';
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitAvis-authentiquesModule')) == true) {
            $this->postProcess();

//            if (!Tools::getValue('AVIS-AUTHENTIQUES_ACCOUNT_email')) {
//                $output .= $this->displayError($this->l('You must enter your email'));
//            }
//
//            if (!Tools::getValue('AVIS-AUTHENTIQUES_ACCOUNT_PASSWORD')) {
//                $output .= $this->displayError($this->l('You must enter your password'));
//            }

            if (!Tools::getValue('AVIS-AUTHENTIQUES_API_KEY')) {
                $output .= $this->displayError($this->l('You must enter an API Key to continue'));
            }

            if (!Tools::getValue('AVIS-AUTHENTIQUES_WIDGET_KEY')) {
                $output .= $this->displayError($this->l('You must enter an Widget Key to continue'));
            }

            if(Tools::getValue('AVIS-AUTHENTIQUES_API_KEY') && Tools::getValue('AVIS-AUTHENTIQUES_WIDGET_KEY')) {
                $output .= $this->displayConfirmation($this->l('The settings have been updated'));
            }
        }

//        $this->requestToken();
//        if(Configuration::get('AVIS-AUTHENTIQUES_BEARER_TOKEN') && Configuration::get('AVIS-AUTHENTIQUES_TOKEN_EXP')) {
//            $output .= $this->displayConfirmation('Authentification API réussie');
//        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output .= $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAvis-authentiquesModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
//                    array(
//                        'col' => 3,
//                        'type' => 'text',
//                        'prefix' => '<i class="icon icon-envelope"></i>',
//                        'desc' => $this->l('Enter a valid email address'),
//                        'name' => 'AVIS-AUTHENTIQUES_ACCOUNT_EMAIL',
//                        'label' => $this->l('Email'),
//                    ),
//                    array(
//                        'col' => 3,
//                        'type' => 'password',
//                        'name' => 'AVIS-AUTHENTIQUES_ACCOUNT_PASSWORD',
//                        'label' => $this->l('Password'),
//                    ),
                    array(
                        'type' => 'text',
                        'name' => 'AVIS-AUTHENTIQUES_API_KEY',
                        'col' => 3,
                        'label' => $this->l('API Key'),
                        'required' => true,
                        'desc' => $this->l('Find your api key in your dashboard')
                    ),
                    array(
                        'type' => 'text',
                        'name' => 'AVIS-AUTHENTIQUES_WIDGET_KEY',
                        'col' => 3,
                        'label' => $this->l('Widget Key'),
                        'required' => true,
                        'desc' => $this->l('Find your widget key in your dashboard')
                    ),
                    array(
                        'type' => 'select',
                        'lang' => true,
                        'label' => $this->l('Position'),
                        'name' => 'AVIS-AUTHENTIQUES_POSITION',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id_option' => 'middle',
                                    'name' => $this->l('Middle')
                                ),
                                array(
                                    'id_option' => 'top',
                                    'name' => $this->l('Top')
                                ),
                                array(
                                    'id_option' => 'bottom',
                                    'name' => $this->l('Bottom')
                                ),
                            ),
                            'id' => 'id_option',
                            'name' => 'name'
                        ),
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'AVIS-AUTHENTIQUES_LIVE_MODE' => Configuration::get('AVIS-AUTHENTIQUES_LIVE_MODE', true),
            'AVIS-AUTHENTIQUES_ACCOUNT_EMAIL' => Configuration::get('AVIS-AUTHENTIQUES_ACCOUNT_EMAIL', 'contact@prestashop.com'),
            'AVIS-AUTHENTIQUES_ACCOUNT_PASSWORD' => Configuration::get('AVIS-AUTHENTIQUES_ACCOUNT_PASSWORD', null),
            'AVIS-AUTHENTIQUES_API_KEY' => Configuration::get('AVIS-AUTHENTIQUES_API_KEY', null),
            'AVIS-AUTHENTIQUES_WIDGET_KEY' => Configuration::get('AVIS-AUTHENTIQUES_WIDGET_KEY', null),
            'AVIS-AUTHENTIQUES_BEARER_TOKEN' => Configuration::get('AVIS-AUTHENTIQUES_BEARER_TOKEN', null),
            'AVIS-AUTHENTIQUES_TOKEN_EXP' => Configuration::get('AVIS-AUTHENTIQUES_TOKEN_EXP', null),
            'AVIS-AUTHENTIQUES_POSITION' => Configuration::get('AVIS-AUTHENTIQUES_POSITION', null),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    public function displayWidget()
    {
        $widgetKey = Configuration::get('AVIS-AUTHENTIQUES_WIDGET_KEY');
        $position = Configuration::get('AVIS-AUTHENTIQUES_POSITION');

        $this->context->smarty->assign(array(
            'widgetKey' => $widgetKey,
            'position' => $position,
        ));
        return $this->display(__FILE__, 'views/templates/front/displayWidget.tpl');
    }

    public function hookActionPaymentConfirmation($paramsAction)
    {
        $orderId = $paramsAction['id_order'];
        $sqlQuery = "SELECT o.id_order, o.reference, c.firstname, c.lastname, c.email FROM " . _DB_PREFIX_ . "orders o LEFT JOIN " . _DB_PREFIX_ . "customer c ON c.id_customer = o.id_customer WHERE o.id_order='$orderId'";
        $order = Db::getInstance()->getRow($sqlQuery);
//        dump($order); die();
        if (!empty($order)) {
//            $exp = Configuration::get('AVIS-AUTHENTIQUES_TOKEN_EXP', null);
//            $now = new DateTime("now");
//            if(empty($exp) || $exp > $now->getTimestamp()) {
//                $this->requestToken();
//            }
            $this->requestAPI('/orders', 'POST', [
                "reference" => $order['reference'],
                "customerEmail" => $order['email'],
                "customerLastName" => $order['lastname'],
                "customerFirstName" => $order['firstname']
            ]);
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
    }

    public function hookDisplayBeforeBodyClosingTag()
    {
//        die('kiri');
        return $this->displayWidget();
    }

    public function hookDisplayFooter()
    {

    }

//    private function requestToken()
//    {
//        $response = $this->requestAPI('/authentication_token', 'POST', [
//            "email" => Configuration::get('AVIS-AUTHENTIQUES_ACCOUNT_EMAIL', null),
//            "password" => Configuration::get('AVIS-AUTHENTIQUES_ACCOUNT_PASSWORD', null),
//        ]);
//
//        $decode = json_decode($response);
//
//        if(isset($decode->code)){
//            $this->context->controller->errors[] = $this->l('Api auth failed, try to reconnect.');
//            return;
//        }
//
//        $token = $decode->token;
//
//        $decodeJWT = $this->decodeJWT($token);
//
//        Configuration::updateValue('AVIS-AUTHENTIQUES_BEARER_TOKEN', $token);
//        Configuration::updateValue('AVIS-AUTHENTIQUES_TOKEN_EXP', $decodeJWT->exp);
//
//    }

    private function requestAPI($url, $method = 'GET', $data = [])
    {
//        $token = Configuration::get('AVIS-AUTHENTIQUES_BEARER_TOKEN', null);
        $apiKey = Configuration::get('AVIS-AUTHENTIQUES_API_KEY', null);

        $curl = curl_init();

        switch ($method)
        {
            case "POST":
                curl_setopt($curl, CURLOPT_POST, 1);

                if ($data)
                    curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
                break;
            case "PUT":
                curl_setopt($curl, CURLOPT_PUT, 1);
                break;
//            default:
//                if ($data)
//                    $url = sprintf("%s?%s", $url, http_build_query($data));
        }


//        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json', 'Authorization: Bearer ' . $token));
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json', 'X-AUTH-API-KEY: ' . $apiKey));

        curl_setopt($curl, CURLOPT_URL, $this->baseURL . $url);
        if (_PS_MODE_DEV_ === true) {
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, FALSE);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
        }
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);

        $result = curl_exec($curl);

        if($result === false)
        {
            echo 'Erreur Curl : ' . curl_error($curl);
            exit;
        }

        curl_close($curl);


        return $result;
    }

//    private function decodeJWT($token)
//    {
//        return json_decode(base64_decode(str_replace('_', '/', str_replace('-','+',explode('.', $token)[1]))));
//    }

}
